/*
    UHU-menu -- output plugin for javascript-enabled browsers
    Copyright (C) 2003  Pozsár Balázs <pozsy@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

int  counter = 0;

static void print (FILE *f, const char *s)
{
	if (s == NULL || *s == '\0') {
		fprintf(f, "\xC2\xA0");
		return;
	}
	while (*s != '\0') {
		if (*s == ' ') {
			fprintf(f, "\xC2\xA0");
			s++;
			continue;
		}
		if (*s == '"')
			fputc('\\', f);
		fputc(*s++, f);
	}
}

static void dump (FILE *f, const struct item *item, const char *lang, int n, const char *name, int offset)
{
	int i;
	fprintf(f, "z = new Array();\n");
	fprintf(f, "t = new Array();\n");
	fprintf(f, "l = new Array();\n");
	fprintf(f, "e = new Array();\n");
	fprintf(f, "g = new Array();\n");
	fprintf(f, "c = new Array();\n");
	for (i = 0; i < item->subitems_nr; i++) {
		fprintf(f, "t[%d] = \"", i);
		print(f, get_value(&item->subitems[i], NULL, "Name", lang));
		fprintf(f, "\";\n");
		fprintf(f, "l[%d] = \"%s\";\n", i, item->subitems[i].x_uhu_menu);
		fprintf(f, "e[%d] = \"", i);
		print(f, get_value(&item->subitems[i], NULL, "Exec", lang));
		fprintf(f, "\";\n");
		fprintf(f, "g[%d] = \"", i);
		print(f, get_value(&item->subitems[i], NULL, "GenericName", lang));
		fprintf(f, "\";\n");
		fprintf(f, "c[%d] = \"", i);
		print(f, get_value(&item->subitems[i], NULL, "Comment", lang));
		fprintf(f, "\";\n");
	}
	fprintf(f, "z[\"text\"] = t;\n");
	fprintf(f, "z[\"load\"] = l;\n");
	fprintf(f, "z[\"exec\"] = e;\n");
	fprintf(f, "z[\"genericname\"] = g;\n");
	fprintf(f, "z[\"comment\"] = c;\n");
	fprintf(f, "z[\"depth\"] = %d;\n", n);
	fprintf(f, "z[\"offset\"] = %d;\n", offset);
	fprintf(f, "data[\"%s\"] = z;\n", name);
	fprintf(f, "\n");
	for (i = 0; i < item->subitems_nr; i++) {
		if (item->subitems[i].issubmenu) {
			dump(f, &item->subitems[i], lang, n+1, item->subitems[i].x_uhu_menu, offset+i);
		}
	}
}

int write_menu_lang (const struct item *item, const char *path,
  const struct language *language, struct keep_files *keep_files)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu%s%s.html", path, lang[0] ? "." : "", lang);
	if ((f = fopen(file, "w")) == NULL) {
		message_e(3, "fopen %s", file);
		return -1;
	}
	if (item->subitems_nr > 0) {
		fprintf(f,
"<html>\n"
"<head>\n"
"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
"<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n"
"<meta name=\"Generator\" content=\"uhu-menu\">\n"
"<title>"
);
		print(f, get_value(item, NULL, "Name", lang));
		fprintf(f,
"</title>\n"
"<style>\n"
"BODY {\n"
"background: #FFFFFF none;\n"
"color: #000000;\n"
"white-space: nowrap;\n"
"}\n"
"TABLE {\n"
"margin: 0px 0px 0px 0px;\n"
"padding: 0px 0px 0px 0px;\n"
"border-spacing: 0px;\n"
"border-style: hidden;\n"
"border-collapse: collapse;\n"
"}\n"
"</style>\n"
"<script>\n"
"var locolor = \"#00ccff\";\n"
"var hicolor = \"#ffcc00\";\n"
"var hi   = new Array();\n"
"var list = new Array();\n"
"var data = new Array();\n"
"\n"
);
		dump(f, item, lang, 0, "/", 0);
		fprintf(f,
"function func(f) {\n"
"	return ((navigator.appVersion.toLowerCase().indexOf('msie') >= 0) ? new Function(f) : f);\n"
"}\n"
"\n"
"function _remove(o) {\n"
"	o.removeChild(o.firstChild);\n"
"}\n"
"\n"
"function _replacechild(o, c) {\n"
"	o.replaceChild(c, o.firstChild);\n"
"}\n"
"\n"
"function setlo(o) {\n"
"	o.setAttribute(\"bgColor\", locolor);\n"
"}\n"
"\n"
"function sethi(o) {\n"
"	o.setAttribute(\"bgColor\", hicolor);\n"
"}\n"
"\n"
"function show(name, depth, exec, genericname, comment) {\n"
"	_replacechild(document.getElementById(\"Exec\"), document.createTextNode(exec));\n"
"	_replacechild(document.getElementById(\"GenericName\"), document.createTextNode(genericname));\n"
"	_replacechild(document.getElementById(\"Comment\"), document.createTextNode(comment));\n"
"	for(i = depth; i < list.length; i++) {\n"
"		if (hi[i]) {\n"
"			for(j = 0; j < hi[i].length; j++)\n"
"				setlo(hi[i][j]);\n"
"			hi[i] = \"\";\n"
"		}\n"
"	}\n"
"	if (document.getElementById(name)) {\n"
"		hi[depth] = new Array();\n"
"		sethi((hi[depth][0] = document.getElementById(name)));\n"
"		sethi((hi[depth][1] = document.getElementById(name + \"/\")));\n"
"	}\n"
"	for(i = depth+1; i < list.length; i++) {\n"
"		if (list[i]) {\n"
"			list[i].style.display = \"none\";\n"
"			_remove(list[i]);\n"
"			list[i] = \"\";\n"
"		}\n"
"	}\n"
"	if (!data[name])\n"
"		return;\n"
"	_table = document.createElement(\"table\");\n"
"	_tbody = document.createElement(\"tbody\");\n"
"	for(i = 0; i < data[name][\"offset\"]; i++) {\n"
"		_td = document.createElement(\"td\");\n"
"		_td.appendChild(document.createTextNode(\"\\u00A0\"));\n"
"		_tr = document.createElement(\"tr\");\n"
"		_tr.appendChild(_td);\n"
"		_tbody.appendChild(_tr);\n"
"	}\n"
"	for(i = 0; i < data[name][\"text\"].length; i++) {\n"
"		_td = document.createElement(\"td\");\n"
"		if (data[name][\"comment\"][i] != \"\xC2\xA0\") {\n"
"			_a = document.createElement(\"a\");\n"
"			_a.appendChild(document.createTextNode(data[name][\"text\"][i]));\n"
"			_a.setAttribute(\"title\", data[name][\"comment\"][i]);\n"
"		} else {\n"
"			_a = document.createTextNode(data[name][\"text\"][i]);\n"
"		}\n"
"		_td.appendChild(_a);\n"
"		_td.setAttribute(\"id\", data[name][\"load\"][i]);\n"
"		_td.setAttribute(\"bgColor\", locolor);\n"
"		_td.setAttribute(\"onmouseover\", func(\"show('\"+data[name][\"load\"][i]+\"', \"+data[name][\"depth\"]+\", '\"+data[name][\"exec\"][i]+\"', '\"+data[name][\"genericname\"][i]+\"', '\"+data[name][\"comment\"][i]+\"');\"));\n"
"		_tr = document.createElement(\"tr\");\n"
"		_tr.appendChild(_td);\n"
"		_td = document.createElement(\"td\");\n"
"		_td.appendChild(document.createTextNode(data[data[name][\"load\"][i]] ? \"\xC2\xA0\xC2\xBB\" : \"\"));\n"
"		_td.setAttribute(\"id\", data[name][\"load\"][i] + \"/\");\n"
"		_td.setAttribute(\"bgColor\", locolor);\n"
"		_td.setAttribute(\"onmouseover\", func(\"show('\"+data[name][\"load\"][i]+\"', \"+data[name][\"depth\"]+\", '\"+data[name][\"exec\"][i]+\"', '\"+data[name][\"genericname\"][i]+\"', '\"+data[name][\"comment\"][i]+\"');\"));\n"
"		_tr.appendChild(_td);\n"
"		_tbody.appendChild(_tr);\n"
"	}\n"
"	_table.appendChild(_tbody);\n"
"	td = document.createElement(\"td\");\n"
"	td.appendChild(_table);\n"
"	document.getElementById(\"_\").appendChild(td);\n"
"	list[data[name][\"depth\"]] = td;\n"
"}\n"
"function start() {\n"
"	show(\"/\", 0, \"\xC2\xA0\", \"\xC2\xA0\", \"\xC2\xA0\");\n"
"}\n"
"</script>\n"
"</head>\n"
"<body onload=\"start()\">\n"
"<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" hspace=\"0\" vspace=\"0\" width=\"100%%\">\n"
"<tr valign=\"top\"><td bgcolor=\"#dddddd\" width=\"1%%\">Exec: </td><td id=\"Exec\" bgcolor=\"#dddddd\">Javascript needed. Please use a javascript-enabled browser.</td></tr></tt>\n"
"<tr valign=\"top\"><td bgcolor=\"#dddddd\">GenericName: </td><td id=\"GenericName\" bgcolor=\"#dddddd\">&nbsp;</td></tr>\n"
"<tr valign=\"top\"><td bgcolor=\"#dddddd\">Comment: </td><td id=\"Comment\" bgcolor=\"#dddddd\">&nbsp;</td></tr>\n"
"</table>"
"<br>\n"
"<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" hspace=\"0\" vspace=\"0\"><tr id=\"_\" valign=\"top\"><td>&nbsp;</td></tr></table>\n"
"</body>\n"
"</html>\n"
);
	}
	fclose(f);
	keep_file(keep_files, strrchr(file, '/') + 1);
	return 0;
}

